from abc import abstractmethod
from typing import Type

from .exception import NoTranslationSegmentError
from .prviders.base_provider import BaseProvider
from .prviders.collinsdictionary import CollinsWeb
from .prviders.lexico import Lexico


class DictionaryFactory:

    def __init__(self, dict_cls: Type[BaseProvider], word: str = ''):
        self.dict_cls = dict_cls
        self.word = word
        self._in_lang = None
        self._target_lang = None

    def __enter__(self, ):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        ...

    @property
    @abstractmethod
    def segment(self) -> str:
        ...

    @property
    @abstractmethod
    def lang_codes(self) -> dict:
        """
        {
            'es': 'spanish',
            'zh': 'chinese',
            'de': 'german',
            'fr': 'french',
            'en': 'english',
        }
        :return:
        """

    @property
    def provider(self) -> BaseProvider:
        return self.dict_cls(self.word, self.segment)

    def search(self, word: str, in_lang: str, target_lang: str) -> dict:
        self.word = word if word else self.word
        self._in_lang = in_lang
        self._target_lang = target_lang
        return self.provider.to_dict()


class CollinsDictionary(DictionaryFactory):
    lang_codes = {
        'es': 'spanish',
        'zh': 'chinese',
        'de': 'german',
        'fr': 'french',
        'en': 'english',
    }

    @property
    def segment(self) -> str:
        if self._in_lang not in self.lang_codes or self._target_lang not in self.lang_codes:
            raise NoTranslationSegmentError(str(self._provider_cls),
                                            self._in_lang, self._target_lang)
        if self._in_lang != 'en' and self._target_lang != 'en':
            raise NoTranslationSegmentError(str(self._provider_cls),
                                            self._in_lang, self._target_lang)
        if self._in_lang == self._target_lang == 'en':
            return 'english'
        return f"{self.lang_codes[self._in_lang]}-{self.lang_codes[self._target_lang]}"

    def __init__(self, *, word: str = ''):
        self._provider_cls = CollinsWeb
        super(CollinsDictionary, self).__init__(self._provider_cls, word=word)

    def en2fr(self, word: str = ''):
        return self.search(word, 'en', 'fr')

    def en2es(self, word: str = ''):
        return self.search(word, 'en', 'es')

    def en2de(self, word: str = ''):
        return self.search(word, 'en', 'de')

    def en2zh(self, word: str = ''):
        return self.search(word, 'en', 'zh')

    def es2en(self, word: str = ''):
        return self.search(word, 'es', 'en')

    def fr2en(self, word: str = ''):
        return self.search(word, 'fr', 'en')

    def de2en(self, word: str = ''):
        return self.search(word, 'de', 'en')

    def zh2en(self, word: str = ''):
        return self.search(word, 'zh', 'en')

    def en(self, word: str = ''):
        return self.search(word, 'en', 'en')


class OxfordDictionary(DictionaryFactory):
    lang_codes = {
        'es': 'spanish',
        'en': 'english',
    }

    @property
    def segment(self) -> str:
        if self._in_lang not in self.lang_codes or self._target_lang not in self.lang_codes:
            raise NoTranslationSegmentError(str(self._provider_cls),
                                            self._in_lang, self._target_lang)
        if self._in_lang == self._target_lang:
            return self._in_lang

        return f"{self._in_lang}-{self._target_lang}"

    def __init__(self, *, word: str = ''):
        self._provider_cls = Lexico
        super(OxfordDictionary, self).__init__(self._provider_cls, word=word)

    def en2es(self, word: str = ''):
        return self.search(word, 'en', 'es')

    def es2en(self, word: str = ''):
        return self.search(word, 'es', 'en')

    def en(self, word: str = ''):
        return self.search(word, 'en', 'en')

    def es(self, word: str = ''):
        return self.search(word, 'es', 'es')



import abc
import re
import unicodedata
from typing import Union, Tuple

import requests
from bs4 import BeautifulSoup, Tag
from requests import Response, HTTPError

from .utils import STD_HEADERS


def _norm_str(s: str):
    return re.sub(r"\s+", ' ', unicodedata.normalize("NFKD", s)).strip()


class Parser:
    to_dict_fields = ()

    def __init__(self, markup: Union[BeautifulSoup, str, Tag]):
        self._markup = markup
        self._bs = None

    @property
    def markup(self) -> Union[BeautifulSoup, str, Tag]:
        return self._markup

    @property
    def bs(self) -> Union[BeautifulSoup, Tag]:
        if not self._bs:
            if isinstance(self.markup, (BeautifulSoup, Tag)):
                self._bs = self.markup
            else:
                self._bs = BeautifulSoup(markup=self.markup, features='html.parser')
        return self._bs

    def select(self, p: str, one=True, text=True):
        if one:
            t = self.bs.select_one(p)
            if t and text:
                return _norm_str(t.text)
            return t
        else:
            ts = self.bs.select(p)
            if ts and text:
                return [_norm_str(t.text) for t in ts]
            return ts

    def get_by_cls(self, name: str, cls: str, **attrs) -> Tag:
        return self.bs.find(name, class_=cls, attrs=attrs)

    def to_dict(self):
        _ = {}

        fields = []
        fields.extend(self.to_dict_fields)
        fields.extend([prop.lower().split('val_')[-1] for prop in dir(self) if prop.startswith("val_")])

        for field in set([f.lower() for f in fields]):
            try:
                val = getattr(self, field, getattr(self, f"val_{field}", None))
                if callable(val):
                    val = val()
                if isinstance(val, str):
                    val = unicodedata.normalize("NFKD", val)
                    # remove brackets
                    m = re.match(r"\((?P<c>.+)\)", val)
                    if m:
                        val = m.group("c")
                if val not in [None, '']:
                    _[field.split('val_')[-1]] = val
            except AttributeError:
                ...
                # _[field.split('val_')[-1]] = None
        return _

    def provider_to_list(self, provider_cls, block_selector: Union[
        str, Tuple[str, dict]
    ]):
        try:
            if isinstance(block_selector, str):
                blocks = self.select(block_selector, one=False, text=False)
            else:
                blocks = self.bs.find_all(
                    block_selector[0],
                    **block_selector[1]
                )
        except AttributeError:
            return []
        return [provider_cls(d).to_dict() for d in blocks]


class WebParser(Parser):

    def __init__(self, word: str):
        self.word = word
        super(WebParser, self).__init__(markup='')
        self._rsp = None

    @property
    @abc.abstractmethod
    def url(self):
        ...

    @property
    def rsp(self) -> Response:
        if not self._rsp:
            self._rsp = requests.get(self.url, headers=STD_HEADERS)
            self._rsp.raise_for_status()
        return self._rsp

    @property
    def json(self) -> dict:
        return self.rsp.json()

    @property
    def markup(self) -> str:
        if not self._markup:
            try:
                self._markup = self.rsp.content.decode()
            except HTTPError:
                self._markup = ''
        return self._markup



import json
import unittest

from web_dict import CollinsDictionary, OxfordDictionary


class CollinsTest(unittest.TestCase):

    def rtest_collins(self):
        with CollinsDictionary(word='hacer') as c:
            self._p(c.es2en())

    def test_oxford(self):
        c = OxfordDictionary()
        self._p(
            c.en('tomorrow')
        )

    def _p(self, c):
        print(json.dumps(c, ensure_ascii=False, indent=4))


if __name__ == '__main__':
    unittest.main()



import re

from .base_provider import BaseProvider
from ..parser import Parser


class _PhraseProvider(Parser):
    to_dict_fields = (
        'phrase',
        'trans',
    )

    @property
    def phrase(self):
        return self.select("span.type-phr > span.orth")

    @property
    def trans(self):
        return self.select("span.cit.type-translation > span.quote")


class _IdiomProvider(Parser):
    to_dict_fields = (
        'orth',
        'trans'
    )

    @property
    def orth(self):
        return self.select('span.orth')

    @property
    def trans(self):
        return self.select("span.cit.type-translation > span")


class _GrammarProvider(Parser):
    to_dict_fields = {
        'content',
        'trans',
    }

    @property
    def content(self):
        content = self.select("span.colloc")
        if content:
            return re.match(r"\[(?P<c>.+)\]", content).group("c")
        return content

    @property
    def trans(self):
        return self.select('span.cit.type-translation')


class _ExampleProvider(Parser):
    to_dict_fields = {
        'sent', 'trans'
    }

    @property
    def sent(self):
        try:
            return re.sub(r'\s+', ' ', self.contents[0]).strip()
        except IndexError:
            return None

    @property
    def trans(self):
        try:
            return re.sub(r'\s+', ' ', "; ".join(self.contents[1:])).strip()
        except IndexError:
            return None

    @property
    def contents(self):
        _ = self.select('span.quote', one=False)
        if len(_) == 2:
            return _
        return [t.text.replace("⇒", "").strip() for t in self.bs.find_all('q')]


class _SenseProvider(Parser):
    to_dict_fields = (
        'exp',
        'examples',
        'syn',
        'grammars',
        'idioms',
        'phrases'
        'senses'
    )

    @property
    def syn(self):
        syn_pattern = re.compile(r"\)?\s*\(=?\s*(?P<c>.+)\)")
        t1 = self.select('span.lbl.type-geo > span.lbl.type-syn', text=False)
        if t1:
            syn = re.match(syn_pattern, t1.text).group("c")
            geo_txt = self.select('span.lbl.type-geo')

            return {
                'syn': syn,
                'geo': re.match(r"\((?P<g>.+)\)\s*\(.+\)", geo_txt).group("g")
            }
        try:
            syn = re.match(syn_pattern, self.select('span.lbl.type-syn')).group("c")
        except:
            syn = ''
        return {'syn': syn, 'geo': None}

    @property
    def exp(self):
        exp = "; ".join(t.text for t in
                        self.bs.find_all('span', class_=re.compile(r'cit\s(cit-)?type-translation'), recursive=False, ))

        if not exp:
            exp = self.select("div.def")
        return exp

    @property
    def idioms(self):
        return self.provider_to_list(
            _IdiomProvider, 'div.re.type-idm'
        )

    @property
    def examples(self):
        return self.provider_to_list(_ExampleProvider, "div.cit.type-example") \
               or self.provider_to_list(_ExampleProvider, "div.cit.cit-type-example")

    @property
    def phrases(self):
        return self.provider_to_list(_PhraseProvider, 'div.type-phr')

    # @property
    # def grammars(self):
    #     return self.provider_to_list(_GrammarProvider, "div.sense")

    def val_senses(self):
        return self.provider_to_list(_SenseProvider, 'div.sense')


class _DefProvider(Parser):
    to_dict_fields = (
        "pos",
        'senses',
        'misc'
    )

    def __init__(self, markup: str):
        super(_DefProvider, self).__init__(markup=markup)

    @property
    def pos(self):
        return self.select('span.pos')

    @property
    def misc(self):
        misc = self.select('span.lbl.type-misc')
        return re.match(r"\((?P<c>.+)\)", misc).group('c') if misc else misc

    @property
    def senses(self):
        senses = [s for s in self.provider_to_list(_SenseProvider,
                                                   ('div', dict(class_='sense', recursive=False)))
                  if any(s.values())]
        if not senses:
            senses = [s for s in self.provider_to_list(_SenseProvider,
                                                       ('li', dict(class_='sense_list_item', recursive=True)))
                      if any(s.values())]
        return senses


class CollinsWeb(BaseProvider):
    to_dict_fields = (
        'head_word',
        'pron',
        'rank',
        'audio',
        'defs'
    )

    @property
    def url(self):
        url = f"https://www.collinsdictionary.com/dictionary/{self.seg}/{self.word}"
        print(f"Requesting {url}")
        return url

    def __init__(self, word: str, seg: str = 'spanish-english'):
        super(CollinsWeb, self).__init__(word, seg)

    @property
    def bs(self):
        bs = super(BaseProvider, self).bs
        return (
                bs.find('div', class_=re.compile(r'cB\scB-def.+'))
                or
                bs.find('div', class_=re.compile(r'cB\scB-t'))
        )

    @property
    def pron(self):
        try:
            return self.select('span.pron.type-') or re.match(r"\((?P<pron>.+)\)",
                                                              self.select("div.cB-h > div > span.pron")).group("pron")
        except TypeError:
            return None

    @property
    def rank(self):
        try:
            return int(self.select('span.word-frequency-img', one=True, text=False)['data-band'])
        except Exception as exc:
            return None

    @property
    def head_word(self):
        try:
            return self.select('h2.h2_entry > span.orth') or self.select('h2.h2_entry')
        except AttributeError:
            return None

    @property
    def audio(self):
        return self.select("a.audio_play_button", text=False).get('data-src-mp3')

    @property
    def defs(self):
        return [s for s in self.provider_to_list(_DefProvider, "div.hom") if any(s.values())]



from distutils.core import setup
from pathlib import Path

from setuptools import find_packages

setup(
    name='web_dict',  # How you named your package folder (MyLib)
    packages=find_packages(),  # Chose the same as "name"
    description='parser class for collinsdictionary.com and lexico.com',
    long_description=Path("readme.md").read_text(),
    long_description_content_type='text/markdown',
    version='0.1.32',  # Start with a small number and increase it with every change you make
    license='agpl-3.0',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    author='Kyle, Hwang',  # Type in your name
    author_email='upday7@163.com',  # Type in your E-Mail
    url='https://github.com/upday7/web_dict',  # Provide either the link to your github or to your website
    keywords=['dictionary', 'spanish', 'english', 'chinese', 'collins', 'oxford', 'lexico'],
    install_requires=[  # I get to this in a second
        'bs4',
        'requests',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from .core.factory import CollinsDictionary, OxfordDictionary
from .core.prviders.collinsdictionary import CollinsWeb
from .core.prviders.lexico import Lexico

LexicoDictionary = OxfordDictionary
__all__ = (CollinsDictionary, LexicoDictionary, OxfordDictionary, Lexico, CollinsWeb)



import json
import unittest

from web_dict.core.prviders.lexico import Lexico


class CollinsTest(unittest.TestCase):

    # def test_spanish_1(self):
    #     c = Lexico('dia',)
    #     self._p(c)

    def test_es_1(self):
        c = Lexico('prueba', 'es')
        self._p(c)

    # def test_english_1(self):
    #     c = Lexico('test', 'en')
    #     self._p(c)

    def _p(self, c):
        print(json.dumps(c.to_dict(), indent=4, ensure_ascii=False))


if __name__ == '__main__':
    unittest.main()
